/* ------------------------------------------------------------------
   Functional (External) Iterator Pattern:
   iterator for console input of a list of cities 
*/

import edu.neu.ccs.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.filter.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;

import java.io.*;
import java.awt.*;

class InConsoleRange implements IRange, ConsoleAware {

    /*-------------------------------------------------------- 
      Member data
      *------------------------------------------------------*/
    protected City c;
    
    protected boolean done = true;
    protected boolean submitted = false;
       
    /*-------------------------------------------------------- 
      Constructor
      *------------------------------------------------------*/
    InConsoleRange() { 

        done = false;
        read();          
    }
        
    /*--------------------------------------------------------
      Methods to implement the IRange interface
      *------------------------------------------------------*/

    public void next() {
        read();            
    }


    public Object current() {
	return c;
    }
    
    public boolean hasMore() {
        return (!done); 
    }
    
    /*--------------------------------------------------------------
      Methods
      *------------------------------------------------------------*/
    
	/* read data for one balloon from the console */
	public City read() {
		try {
			c = new City(
					console.in.requestInt("Zip code: "),
					console.in.requestString("City name: "),
					console.in.requestString("State: "),
					requestDegrees("Longitude (degrees west): "),
					requestDegrees("Latitude (degrees north): "));
		} catch (CancelledException e) {
			console.out.println("End of input");
			done = true;
		}
		return c;
	}  
    

	public double requestDegrees(String s) throws CancelledException {
		XDouble temp = new XDouble();
		StringableFilter degFilter = new RangeFilter.Double(-180.0, 180.0);
		
		boolean more = console.in.reading(s, (Stringable) temp, degFilter);
		if (more)
			return temp.doubleValue();
		else
			done = true;
		throw new CancelledException();
	}
	
}
