////////////////////////////////////////////////////////////////
//
// Assignment for week 9.
//
////////////////////////////////////////////////////////////////

import java.util.Comparator;
import java.util.Vector;

////////////////////////////////////////////////////////////////
//
// ListSortAlgorithms
//
////////////////////////////////////////////////////////////////

public class InsertListSort implements SortAlgorithm{
	
	public AList source;
	public AList result;
	
	public InsertListSort(){		
		source = new MTList();
		result = new MTList();
	}

	public InsertListSort(IRange it){		
		initializeDataSet(it);
		result = new MTList();
	}

	// Stores the objects generated by the given IRange into 
	// some local data structure, but does not sort them.
	//
	// This method must be called before any of the others.
	// This method may be called more than once, in which
	// case the objects that were generated for a previous
	// call are discarded.

	public void initializeDataSet(IRange it){
		source = new MTList();
		
		for (; it.hasMore(); it.next() ) {
			source = new ConsList( it.current(), source);			
		}		
	}

	// Sorts the objects into some internal data structure without
	// changing the local data structure that holds the unsorted
	// objects.
	//
	// (Thus a destructive sort algorithm must first perform a
	// shallow copy of the objects to be sorted into the data
	// structure that will be destroyed by the sort() method.)
	//
	// Repeated calls to this method should repeat all the steps
	// of the first call, because this method will be timed to
	// measure the performance of different sort algorithms.
	//
	// This method shall not be called until after this object's
	// initializeDataSet(IRange) method has been called.  This
	// method shall be called before this object's
	// isSorted(Comparator) method is called.

	// insertion sort
	public void sort(Comparator cmp) {
		result = insertSort(source, cmp);
	}
	
	// helper method to do recursive insertion sort
	protected AList insertSort(AList list, Comparator cmp){
		if (list.isEmpty())
			return new MTList();
		else
			return insert( ((ConsList)list).first(), 
			               insertSort(((ConsList)list).rest(), cmp),
			               cmp );
	}
	
	// helper method to insert the given object into a sorted list
	public AList insert(Object obj, AList list, Comparator cmp){
		if (list.isEmpty())
			return new ConsList(obj, list);
		else if (cmp.compare(obj, ((ConsList)list).first()) < 0 )
			return new ConsList(obj, list);
		else
			return new ConsList( ((ConsList)list).first(), 
								 insert(obj, ((ConsList)list).rest(), cmp) );
	}
	

	// Returns true if the sorted objects are in order according
	// to the given Comparator, which is not necessarily the one
	// that was used to perform the sort.
	//
	// This method shall not be called until after this object's
	// sort(Comparator) has been called.

	public boolean isSorted(Comparator cmp){
		ListRange it = new ListRange(result);
		
		boolean sorted = true;
		
		// empty list is sorted
		if (!it.hasMore())
			return true;
		
		Object firstOne = it.current();
		
		// list with one object is sorted
		// otherwise, make the next item thbasis for comparison
		for (it.next(); it.hasMore(); it.next() ){
			if (cmp.compare(it.current(), firstOne) < 0)	
				return false;
			else 
			firstOne = it.current();
		}
		
		// all is fine if the loop finishes
		return true;
	
	}

	// Returns an IRange that generates the sorted objects, in
	// sorted order.
	//
	// This method shall not be called until after this object's
	// sort(Comparator) has been called.
	// If this method is called more than once, it shall return
	// a freshly created IRange for each call.

	public IRange result(){
		return new ListRange(result);
	}
}

