////////////////////////////////////////////////////////////////
//
// Assignment for week 9.
//
////////////////////////////////////////////////////////////////

import java.util.Comparator;
import java.util.Iterator;
import java.util.Vector;

////////////////////////////////////////////////////////////////
//
// VectorSortAlgorithms
//
////////////////////////////////////////////////////////////////

public class InsertVectorSort implements SortAlgorithm{
	
	public Vector source;
	public Vector result;
	
	public InsertVectorSort(){		
		source = new Vector();
		result = new Vector();
	}

	public InsertVectorSort(IRange it){		
		initializeDataSet(it);
		result = new Vector();
	}

	// Stores the objects generated by the given IRange into 
	// some local data structure, but does not sort them.
	//
	// This method must be called before any of the others.
	// This method may be called more than once, in which
	// case the objects that were generated for a previous
	// call are discarded.

	public void initializeDataSet(IRange it){
		source = new Vector();
		
		for (; it.hasMore(); it.next() ) {
			source.addElement(it.current());			
		}		
	}

	// Sorts the objects into some internal data structure without
	// changing the local data structure that holds the unsorted
	// objects.
	//
	// (Thus a destructive sort algorithm must first perform a
	// shallow copy of the objects to be sorted into the data
	// structure that will be destroyed by the sort() method.)
	//
	// Repeated calls to this method should repeat all the steps
	// of the first call, because this method will be timed to
	// measure the performance of different sort algorithms.
	//
	// This method shall not be called until after this object's
	// initializeDataSet(IRange) method has been called.  This
	// method shall be called before this object's
	// isSorted(Comparator) method is called.

	// insertion sort
	public void sort(Comparator cmp) {
		this.result = new Vector();
	
		int n = this.source.size();

		for (int i = 0; i < n; i = i + 1) {
			Object c = this.source.elementAt (i);
			int m = this.result.size();
			int j = 0;
			for ( ;
			  (j < m) && (cmp.compare (c, this.result.elementAt (j)) > 0);
			  j = j + 1) {
			}
			this.result.add (j, c);
		}
	}


	// Returns true if the sorted objects are in order according
	// to the given Comparator, which is not necessarily the one
	// that was used to perform the sort.
	//
	// This method shall not be called until after this object's
	// sort(Comparator) has been called.

	public boolean isSorted(Comparator cmp){
		VectorRange it = new VectorRange(result);
		
		boolean sorted = true;
		
		// empty list is sorted
		if (!it.hasMore())
			return true;
		
		Object firstOne = it.current();
		
		// list with one object is sorted
		// otherwise, make the next item thbasis for comparison
		for (it.next(); it.hasMore(); it.next() ){
			if (cmp.compare(it.current(), firstOne) < 0)	
				return false;
			else 
			firstOne = it.current();
		}
		
		// all is fine if the loop finishes
		return true;
	
	}

	// Returns an IRange that generates the sorted objects, in
	// sorted order.
	//
	// This method shall not be called until after this object's
	// sort(Comparator) has been called.
	// If this method is called more than once, it shall return
	// a freshly created IRange for each call.

	public IRange result(){
		return new VectorRange(result);
	}
}

