////////////////////////////////////////////////////////////////
//
// Assignment for week 9.
//
////////////////////////////////////////////////////////////////

import java.util.Comparator;
import java.util.Vector;

////////////////////////////////////////////////////////////////
//
// ListSortAlgorithms
//
////////////////////////////////////////////////////////////////

public class QuickListSort implements SortAlgorithm{
	
	public AList source;
	public AList result;
	
	public QuickListSort(){		
		source = new MTList();
		result = new MTList();
	}

	public QuickListSort(IRange it){		
		initializeDataSet(it);
		result = new MTList();
	}

	// Stores the objects generated by the given IRange into 
	// some local data structure, but does not sort them.
	//
	// This method must be called before any of the others.
	// This method may be called more than once, in which
	// case the objects that were generated for a previous
	// call are discarded.

	public void initializeDataSet(IRange it){
		source = new MTList();
		
		for (; it.hasMore(); it.next() ) {
			source = new ConsList( it.current(), source);			
		}		
	}

	// Sorts the objects into some internal data structure without
	// changing the local data structure that holds the unsorted
	// objects.
	//
	// (Thus a destructive sort algorithm must first perform a
	// shallow copy of the objects to be sorted into the data
	// structure that will be destroyed by the sort() method.)
	//
	// Repeated calls to this method should repeat all the steps
	// of the first call, because this method will be timed to
	// measure the performance of different sort algorithms.
	//
	// This method shall not be called until after this object's
	// initializeDataSet(IRange) method has been called.  This
	// method shall be called before this object's
	// isSorted(Comparator) method is called.

	// quick sort
	public void sort(Comparator cmp) {
		result = quickSort(source, cmp);
	}
	
	public AList quickSort(AList input, Comparator cmp) {
		
		if (input.isEmpty() || input.rest().isEmpty())
			return input;
		else {
			// Partition step.
			AList less = new MTList();			// left partition
			AList more = new MTList();			// right partition
			Object pivot = input.first();		// the pivot

			// because neither partition contains the pivot,
			// the resulting subproblems are guaranteed to be smaller than the original
			// assuring termination of the algorithm !!!!!!!!
			// ADDING THE PIVOT TO less OR more GUARANTEES AN INFINITE LOOP
			// IF A PIVOT HAS A DUPLICATE IN THE DATA SET
						
			input = input.rest();				// rest of the list to sort
			
			while (!(input.isEmpty())) {
				Object x = input.first();
				input = input.rest();
				if (cmp.compare(x, pivot) < 0)
					less = new ConsList(x, less);
				else
					more = new ConsList(x, more);
			}
			
			// sort the left and right sides
			less = this.quickSort(less, cmp);
			more = this.quickSort(more, cmp);
			
			// combine them into one list
			return less.append(new ConsList(pivot, more));
		}
	}


	// Returns true if the sorted objects are in order according
	// to the given Comparator, which is not necessarily the one
	// that was used to perform the sort.
	//
	// This method shall not be called until after this object's
	// sort(Comparator) has been called.

	public boolean isSorted(Comparator cmp){
		ListRange it = new ListRange(result);
		
		boolean sorted = true;
		
		// empty list is sorted
		if (!it.hasMore())
			return true;
		
		Object firstOne = it.current();
		
		// list with one object is sorted
		// otherwise, make the next item thbasis for comparison
		for (it.next(); it.hasMore(); it.next() ){
			if (cmp.compare(it.current(), firstOne) < 0)	
				return false;
			else 
			firstOne = it.current();
		}
		
		// all is fine if the loop finishes
		return true;
	
	}

	// Returns an IRange that generates the sorted objects, in
	// sorted order.
	//
	// This method shall not be called until after this object's
	// sort(Comparator) has been called.
	// If this method is called more than once, it shall return
	// a freshly created IRange for each call.

	public IRange result(){
		return new ListRange(result);
	}
}

