////////////////////////////////////////////////////////////////
//
// Assignment for week 9.
//
////////////////////////////////////////////////////////////////

import java.util.Comparator;
import java.util.Vector;

////////////////////////////////////////////////////////////////
//
// SortAlgorithm
//
////////////////////////////////////////////////////////////////

interface SortAlgorithm {

    // Stores the objects generated by the given IRange into 
    // some local data structure, but does not sort them.
    //
    // This method must be called before any of the others.
    // This method may be called more than once, in which
    // case the objects that were generated for a previous
    // call are discarded.

    public void initializeDataSet(IRange ir);

    // Sorts the objects into some internal data structure without
    // changing the local data structure that holds the unsorted
    // objects.
    //
    // (Thus a destructive sort algorithm must first perform a
    // shallow copy of the objects to be sorted into the data
    // structure that will be destroyed by the sort() method.)
    //
    // Repeated calls to this method should repeat all the steps
    // of the first call, because this method will be timed to
    // measure the performance of different sort algorithms.
    //
    // This method shall not be called until after this object's
    // initializeDataSet(IRange) method has been called.  This
    // method shall be called before this object's
    // isSorted(Comparator) method is called.

    public void sort(Comparator cmp);

    // Returns true if the sorted objects are in order according
    // to the given Comparator, which is not necessarily the one
    // that was used to perform the sort.
    //
    // This method shall not be called until after this object's
    // sort(Comparator) has been called.

    public boolean isSorted(Comparator cmp);

    // Returns an IRange that generates the sorted objects, in
    // sorted order.
    //
    // This method shall not be called until after this object's
    // sort(Comparator) has been called.
    // If this method is called more than once, it shall return
    // a freshly created IRange for each call.

    public IRange result();
}

