/* TestSuite.java  16 April 2003 */

import edu.neu.ccs.*;
import edu.neu.ccs.util.*;
import edu.neu.ccs.filter.*;
import edu.neu.ccs.gui.*;

import java.awt.*;
import java.math.*;
import java.util.*;

public class TestSuite extends JPFalt {
    public static void main(String[] args) { new TestSuite(); }
    
	/*-----------------------------------------------------------------------
	 * Objects to be used in tests
	 *----------------------------------------------------------------------- */
	
	String s1 = "one";
	String s2 = "two";
	String s3 = "three";
	String s4 = "four";
	String s5 = "five";

	AList mt1 = new MTList();
	AList mt2 = new MTList();

	AList l1 = new ConsList (s1, mt1);
	AList l2 = new ConsList (s2, mt2);

	AList lst5 = new ConsList (s5, mt2);
	AList lst4 = new ConsList (s4, lst5);
	AList lst3 = new ConsList (s3, lst4);
	AList lst2 = new ConsList (s2, lst3);
	AList lst1 = new ConsList (s1, lst2);

	AList list5 = new ConsList (s5, mt1);
	AList list4 = new ConsList (s4, list5);
	AList list3 = new ConsList (s4, list4);  // note difference!
	AList list2 = new ConsList (s2, list3);
	AList list1 = new ConsList (s1, list2);

	Comparator ic1 = new Comparator () {
		public int compare (Object x, Object y) {
			return ((String) x).compareTo (y);
		}
	};
	
	Comparator ic2 = new Comparator () {
		public int compare (Object x, Object y) {
			return ((String) y).compareTo (x);
		}
	};
	
	// zip codes are encoded as String, because 01944 is not a valid octal number
	City tuscon = new City("85745", "Tucson", "AZ", 111.017907, 32.243359);
	City madriver = new City("95552", "Mad River", "CA", 123.413994, 40.352352);        
	City vail = new City("81657", "Vail", "CO", 106.463454, 39.623793);
	City burlington = new City("06013", "Burlington", "CT", 72.944386, 41.757296);
	City atlanta1 = new City("30319", "Atlanta", "GA", 84.335091, 33.868728);
	City atlanta2 = new City("30324", "Atlanta", "GA", 84.354867, 33.820609);
	City atlanta3 = new City("30326", "Atlanta", "GA", 84.358232, 33.848168);
	City lexGA = new City("30648", "Lexington", "GA", 83.085809, 33.879913);
	City newtonIA = new City("50208", "Newton", "IA", 93.045456, 41.699173);
	City manchMA = new City("01944", "Manchester", "MA", 70.767434, 42.57963);
	City lexMA = new City("02173", "Lexington", "MA", 71.225916, 42.445384);
	City newtonMA = new City("02159", "Newton", "MA", 71.191839, 42.318889);
	City manchNH1 = new City("03102", "Manchester", "NH", 71.488433, 42.99442);
	City manchNH2 = new City("03103", "Manchester", "NH", 71.449325, 42.965563);
	City boston = new City("02115", "Boston", "MA", 71.092215, 42.342706);
	
	AList cities = new ConsList(boston,
					new ConsList(manchMA,
					 new ConsList(manchNH1,
					  new ConsList(newtonIA,
					   new ConsList(newtonMA,
					    new ConsList(manchNH2, new MTList()))))));

	

    /*----------------------------------------------------------------------- 
     * Test city constructors
     *----------------------------------------------------------------------- */
    void TestCity(){
	
        City nyc = new City();
        println(nyc);
        
        City boston = new City(02115, "Boston", "MA", 71.092215, 42.342706);
        println(boston);
        
        City bos = new City(boston);
        println(boston);
        
        println("Testing zip code converter");
        expected(12345);
        actual(boston.decimal("12345"));
        
        expected(2145);
        actual(boston.decimal("02145"));
    }
    
    /*----------------------------------------------------------------------- 
     * Test GUI iterator for input of list of cities
     *----------------------------------------------------------------------- */
    void TestInGuiRange(){
        AList cities = new MTList();
        
        for (IRange it = new InGuiRange();
             it.hasMore();
             it.next()){
            cities = new ConsList((City)it.current(), cities);
            println("Next in City: " + ((ConsList)cities).first);
        }
        
        println("Done with gui input iterator.");
    }

	/*----------------------------------------------------------------------- 
	 Test file iterator for input of list of cities
	------------------------------------------------------------------------- */
	void TestInFileRange() {

		println("\nTesting the file input iterator\n");

		City[] cities = new City[20];
		int i = 0;

		for (IRange it = new InFileRange(); it.hasMore(); it.next()) {

			cities[i] = (City) it.current();
			println("Next in City: " + cities[i]);
			i++;

		}

		println("Done with file input iterator.");
	}

    /*----------------------------------------------------------------------- 
     * Test buffered file iterator for input of list of cities
     *----------------------------------------------------------------------- */
    void TestInFileRangeBuffered(){
        int count = 0;
        
        for (IRange it = new InFileRangeBuffered();
             it.hasMore();
             it.next()){
	    City current = (City)it.current();
	    if (count < 50)
		println("Next in City: " + current);
	    else if (count == 50) 
		println("Not displaying the rest of the cities.");
            
	    count++;
            
        }
        println("Read " + count + " cities from file input iterator.");
    }

    /*-----------------------------------------------------------------------
     * Test console iterator for input of list of cities
     *----------------------------------------------------------------------- */
    void TestInConsoleRange(){
    
        City[] cities = new City[20];
        int i = 0;
        
        for (InConsoleRange it = new InConsoleRange();
             it.hasMore();
             it.next()){
        
            println("loop");
            cities[i] = (City)it.current();
            
            println(cities[i]);
            i++;
        }
    }

	/*----------------------------------------------------------------------- 
	Test file output iterator
	------------------------------------------------------------------------- */
	void TestOutFileRange() {

		println("\nTesting the file output iterator\n");

		println("The file will have the following cities:");
		
		City[] cities = { tuscon, madriver, newtonIA, newtonMA, atlanta1,
			              atlanta2, lexGA, lexMA, vail, boston};

		for (IRange it = new OutFileRange(cities); it.hasMore(); it.next()) {

			println("\nNext City written to the file:");
			println((City) it.current());
		}

		println("Done with file output iterator.");

	}


    /*-----------------------------------------------------------------------
     * Utility methods
     *----------------------------------------------------------------------- */

    /** convert a String to int - if possible */
	protected int intValue(String s){
		int n = 0;
		try{
			n = (new XInt(s)).intValue();
		}
		catch(java.text.ParseException e){
			println("Cannot convert " + s + " to integer.");
		}
		return n;
	}

	
	/*-----------------------------------------------------------------------
	 * Test InsertListSort, InsertVectorSort, QuickListSort, QuickVectorSort
	 *----------------------------------------------------------------------- */
	void SortAlgorithmsTest(){
		testHeader("sort method for in the class SortAlgorithm");
		
		println("Choose one of the following algorithms:");
		println("Insertion sort over lists   [1]");
		println("Insertion sort over vectors [2]");
		println("Quick sort     over lists   [3]");
		println("Quick sort     over vectors [4]");
		int i = console.in.demandInt("Algortihm selection:", new RangeFilter.Long(1, 4));
		
		SortAlgorithm[] sortalgos = {new InsertListSort(new ListRange(mt1)),
						 			 new InsertVectorSort(new ListRange(mt1)),
									 new QuickListSort(new ListRange(mt1)),
									 new QuickVectorSort(new ListRange(mt1))};
		
		// run the selected algorithm							 
		SortAlgorithm sortalgo = sortalgos[i - 1];
		
		println("Choose one of the following comparators:");
		println("Zip code comparator            [1]");
		println("City name and state comparator [2]");
		i = console.in.demandInt("Comparator selection:", new RangeFilter.Long(1, 2));
		
		Comparator[] cmps = {new ZipComparator(),
							 new NameComparator()};
			
		// use the selected comparator							 
		Comparator cmp = cmps[i - 1];

		// test the empty list
		sortalgo.sort (cmp);
		
		expected(true);
		actual(sortalgo.isSorted(cmp));
		
		// test a list of length 5
		sortalgo.initializeDataSet(new ListRange(cities));
			
		sortalgo.sort(cmp);
		expected(true);
		actual(sortalgo.isSorted(cmp));
	
		for (IRange it = sortalgo.result(); it.hasMore(); it.next()){
			println(it.current());
		}
		
		/////// run the test with the same algo and data, new comparator
		println("Select another comparator for the same data and algorithm:");			
		println("Choose one of the following comparators:");
		println("Zip code comparator            [1]");
		println("City name and state comparator [2]");
		i = console.in.demandInt("Comparator selection:", new RangeFilter.Long(1, 2));
		
		cmp = cmps[i - 1];
		sortalgo.sort(cmp);
		expected(true);
		actual(sortalgo.isSorted(cmp));
	
		for (IRange it = sortalgo.result(); it.hasMore(); it.next()){
			println(it.current());
		}

	}

	

	/*-----------------------------------------------------------------------
	 * Timer Test - press button several times for elapsed time in milliseconds
	 *----------------------------------------------------------------------- */
	public void timerTest(){
		println("The time is: " + (new Date()).getTime());	
	}
	
	/*-----------------------------------------------------------------------
	 * Sample test of timing of insertion sort
	 *----------------------------------------------------------------------- */	
	public void testTimerTest(){
		
		InsertListSort ils = new InsertListSort();
		
		AList[] datasets = {cities, cities, cities};		
		
		TimerTests tt = new TimerTests(datasets);
		
		println("Sort timer: \n");
		tt.runTests();
		println("List is sorted: " + ils.isSorted(new ZipComparator()));

		int[] sizes = {100, 500, 1000};
		TimerTests tt1 = new TimerTests(new InFileRangeBuffered(), sizes);
//		tt.readData(new InFileRangeBuffered());
//		tt.initializeDataSets(sizes);
				
		println("Sort timer: \n");
		tt1.runTests();
		println("List is sorted: " + ils.isSorted(new NameComparator()));
		
	}

	/*-----------------------------------------------------------------------
	 * Test the method which initializes the data sets from a file
	 *----------------------------------------------------------------------- */
	public void testDataSets(){
		
		// build test data of sizes 5, 10, and 15
		int[] sizes = {5, 10, 15};
		
		AList[] ds = initializeDataSets(new InFileRangeBuffered(), sizes);

		println("Test data sets:");
		for (int i = 0; i < ds.length; i = i + 1){
			println("ds[" + i + "]:");
			for (IRange it = new ListRange(ds[i]); it.hasMore(); it.next()){
				println(it.current());
			}			
			println("Set " + i + " is empty: " + ds[i].isEmpty());
		}
	}
	
	/*-----------------------------------------------------------------------
	 * Helper method which initializes the data sets for the timer tests
	 *----------------------------------------------------------------------- */
	protected AList[] initializeDataSets(IRange it, int sizes[]){
		
		ArrayList alldata = readData(it);   // contains all data available
		AList tempList = new MTList();		// used to build each data set
		int num = sizes.length;				// number of data sets to be built
		AList[] datasets = new AList[num];	// array of the resulting data sets
		
		// initialize all resulting data sets to empty lists
		for (int i = 0; i < num; i = i + 1)
			datasets[i] = new MTList();
		
		int length = alldata.size();		// the langth of all data available	
		int start;							// we start reading data at this index
		int finish;							// finish reading data before this index

		// build each data set
		for (int dsi = 0; dsi < num; dsi = dsi + 1){
			
			// if there is enough data, start the data set at random index
			if (length > sizes[dsi])
				start = MathUtilities.randomInt(0, length - sizes[dsi]);
			else start = 0;
				
			println("\nThe first data set starts at: " + start);
			println("The size should be: " + sizes[dsi]);
			
			// make sure you do not go over the length of available data
			finish = Math.min(length, start + sizes[dsi]);
				
			// copy the needed amount of data from all available data
			for (int i = start;	i < finish; i = i + 1)
					tempList = new ConsList(alldata.get(i), tempList);
					
			datasets[dsi] = tempList;
			tempList = new MTList();
			}		
				
		return datasets;
	}


	/*-----------------------------------------------------------------------
	 * Test the method which reads data into an ArrayList
	 *----------------------------------------------------------------------- */
	public void testReadData(){
		ArrayList al = readData(new ListRange(cities));
		println("ArrayList size is: " + al.size());	
		for (int i = 1; i < al.size(); i = i + 1){
			println("Item[" + i + "]: " + al.get(i));	
		}
	}

	/*-----------------------------------------------------------------------
	 * Helper method which reads data from a given iterator into an ArrayList
	 *----------------------------------------------------------------------- */
	protected ArrayList readData(IRange it){
		ArrayList alldata = new ArrayList();
		for(;
			it.hasMore();
			it.next()){
			alldata.add(it.current());		
		}
		println(alldata.size() + " data items are available");
		return alldata;		
	}
	
	/*-----------------------------------------------------------------------
	 * Student Tests
	 *----------------------------------------------------------------------- */

}

