import edu.neu.ccs.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.util.*;

import java.io.*;
import java.util.*;

class TimerTests implements ConsoleAware{

	// an array of sorting algorithms to test
	SortAlgorithm[] sas = {new InsertListSort(), 
		                   new InsertVectorSort(), 
						   new QuickListSort(), 
		                   new QuickVectorSort()};
	
	// an array of algorithm labels
	String[] algnames = {"Insert/List", 
                         "Insert/Vector", 
                         "Quicksort/List", 
                         "QuickVectorSort"};

	// an array of comparators to use
	Comparator[] cmps = {new NameComparator(), 
		                 new ZipComparator()};
	
	// an array of comparator labels
	String[] cmpnames = {"NameComparator", 
                         "ZipComparator"};
	
	// data sets of sizes 100, 1000, 10000 used in tests
	AList[] datasets;
	
	// original data set of all cities available
	ArrayList alldata;
	
	// records of the start and end time
	long startTime;			// time of the start of the test
	long endTime;			// time of the end of the test

	// start the timer tests with the given test data sets
	TimerTests(AList[] datasets){
		this.datasets = datasets;		
	}
	
	// start the timer tests with the given test data collection
	// and selected sizes of test data sets
	TimerTests(IRange it, int sizes[]){
		readData(it);
		initializeDataSets(sizes);
	}
	

	// allow the user to select a different set of test data
	public void initializeDataSets(int sizes[]){
		AList tempList = new MTList();
		
		int length = alldata.size();		
		int start;
		datasets = new AList[sizes.length];
		int maxno = sizes.length;
		
		for (int dsi = 0; dsi < maxno; dsi = dsi + 1){
				start = MathUtilities.randomInt(0, length - sizes[dsi]);

				for (int i = start;	i < start + sizes[dsi]; i = i + 1)
					tempList = new ConsList(alldata.get(i), tempList);
					
				datasets[dsi] = tempList;
				tempList = new MTList();
		}				
	}

	// create the base data set for all tests
	public void readData(IRange it){
		alldata = new ArrayList();
		for(;
			it.hasMore();
			it.next()){
			alldata.add(it.current());		
		}
	}

	// run timing tests for all comparators, all algorithms, and all data sets
	public void runTests(){
	
		// for each comparator
		for (int c = 0; c < cmps.length; c = c + 1){	
			console.out.println("Using " + cmpnames[c] + ":");
		
			// for each algorithm
			for (int a = 0; a < sas.length; a = a + 1){
				console.out.println(algnames[a]);
		
				int s = 100;
				// for each data set
				for(int i = 0; i < datasets.length; i = i + 1){ 
			
					sas[a].initializeDataSet(new ListRange(datasets[i]));
					startTime = (new Date()).getTime();		
					sas[a].sort(cmps[c]);
					endTime = (new Date()).getTime();
					console.out.println("Data set[" + i + "] size = " + s + "  time "+ (endTime - startTime));					
					s = s * 10;
				}
				console.out.println(".............");
			}
			console.out.println();
		}
	}

}